using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;
using nToolbox.Collections;
using NUnit.Framework;

namespace nToolbox.Core.Test.Collections
{
	[TestFixture]
	public class TestBag
	{
		private Bag<string> deck;
		private string[] cards;
		private const string B = "B";
		private const string A = "A";

		[TestFixtureSetUp]
		public void FixtureSetUp()
		{
			cards = new string[] {"2", "3", "4", "5", "6", "7", "8", "9", "0", "J", "Q", "K", "A"};
		}

		[SetUp]
		public void SetupTestBag()
		{
			deck = new Bag<string>();
			deck.AddRange(cards, 4); // create a deck of cards.
		}

		[Test]
		public void TestICollectionInterface()
		{
			Assert.AreEqual(52, deck.Count);
			Assert.IsTrue(deck.Contains(A));
			Assert.IsFalse(deck.Contains(B));
			Assert.IsFalse(deck.IsReadOnly);

			string[] array = new string[52];
			bool ok = false;
			try
			{
				deck.CopyTo(array, 1);
			}
			catch (IndexOutOfRangeException)
			{
				ok = true;
			}
			Assert.IsTrue(ok);
			deck.CopyTo(array, 0);
			Assert.IsTrue(Array.TrueForAll(array, NotNull));
			deck.Clear();
			Assert.AreEqual(0, deck.Count);
			deck.Add(B);
			Assert.IsTrue(deck.Contains(B));
			Assert.IsFalse(deck.Contains(A));
			Assert.AreEqual(1, deck.Count);
			deck.Add(B);
			Assert.IsTrue(deck.Contains(B));
			Assert.AreEqual(2, deck.Count);

			Assert.IsFalse(deck.Remove(A));
			Assert.IsTrue(deck.Remove(B));
			Assert.AreEqual(1, deck.Count);
			Assert.IsTrue(deck.Remove(B));
			Assert.IsFalse(deck.Remove(B));
			Assert.AreEqual(0, deck.Count);
		}

		private static bool NotNull(string value)
		{
			return value != null;
		}

		[Test]
		public void TestIEnumerableInterfaces()
		{
			StringBuilder build = new StringBuilder();
			foreach (string card in deck)
			{
				build.Append(card);
			}
			Assert.AreEqual(52, build.Length);

            // Test the non-generic IEnumerable interface, logically the same as the generic.
            string first = build.ToString();
			build = new StringBuilder();

			// Explictly get the IEnumerable interface.
			IEnumerable deckAsIEnumerable = deck;
			foreach (object card in deckAsIEnumerable)
			{
				build.Append(card.ToString());
			}
			Assert.AreEqual(52, build.Length);
            StringAssert.IsMatch(first, build.ToString());
		}

		[Test]
		public void TestClearRemove()
		{
			Assert.AreEqual(52, deck.Count);
			Assert.AreEqual(4, deck.GetCount(A));
			Assert.AreEqual(13, deck.UniqueCount);

			Assert.IsTrue(deck.Remove(A, 2));
			Assert.AreEqual(50, deck.Count);
			Assert.AreEqual(2, deck.GetCount(A));
			Assert.AreEqual(13, deck.UniqueCount);

			Assert.IsFalse(deck.Remove(B, 1));
			Assert.IsFalse(deck.RemoveAll(B));
			Assert.IsTrue(deck.RemoveAll(A));
			Assert.AreEqual(48, deck.Count);
			Assert.AreEqual(0, deck.GetCount(A));
			Assert.AreEqual(12, deck.UniqueCount);
			Assert.IsTrue(deck.RemoveAll("2"));
			Assert.AreEqual(44, deck.Count);
			deck.Clear();
			Assert.AreEqual(0, deck.Count);
		}

		[Test]
		public void TestAdds()
		{
			Bag<string> bag = new Bag<string>();
			Assert.AreEqual(0, bag.Count);
			bag.Add(B, 2);
			Assert.AreEqual(2, bag.Count);
			bag.Add(A, 3);
			Assert.AreEqual(5, bag.Count);
			bag.Add(B, 4);
			Assert.AreEqual(9, bag.Count);

			bag.Clear();
			Assert.AreEqual(0, bag.Count);
			bag.AddRange(cards);
			Assert.AreEqual(13, bag.Count);
			bag.AddRange(cards, 3);
			Assert.AreEqual(52, bag.Count);

			bag.Clear();
			bag.AddRange(new string[] {A, B, B, B, A, A, A, "C", "D"});
			Assert.AreEqual(9, bag.Count);
			Assert.AreEqual(4, bag.UniqueCount);
		}

		[Test]
		public void TestToListsAndUniqueValues()
		{
			List<string> all = deck.ToList();
			List<string> unique = deck.ToUniqueList();

			Assert.AreEqual(52, all.Count);
			Assert.AreEqual(13, unique.Count);

			all.Clear();
			foreach (string item in deck.UniqueValues())
			{
				all.Add(item);
			}

			Assert.AreEqual(13, all.Count);
			Assert.AreEqual(unique.ToArray(), all.ToArray());
		}

		[Test]
		public void TestRandomItem()
		{
			StringBuilder builder = new StringBuilder();
			deck.SeedRandom(1234);
			for (int i = 0; i < 100; i++)
				builder.Append(deck.GetRandomItem(false));
			string first = builder.ToString();
			builder.Remove(0, builder.Length);
			deck.SeedRandom(1234);
			for (int i = 0; i < 100; i++)
				builder.Append(deck.GetRandomItem(false));
            StringAssert.IsMatch( first, builder.ToString() );

			builder.Remove(0, builder.Length);
			deck.SeedRandom(123);
			for (int i = 0; i < 100; i++)
				builder.Append(deck.GetRandomItem(false));
			Assert.IsFalse(first.Equals(builder.ToString()));

			builder.Remove(0, builder.Length);
			while (deck.Count > 0)
				builder.Append(deck.GetRandomItem(true));
			Assert.AreEqual(52, builder.Length);
		}
	}
}