using System;
using System.IO;
using System.Security.Cryptography;
using nToolbox.Security;
using NUnit.Framework;

namespace nToolbox.Core.Test
{
	/// <summary>
	/// Tests are for the generic version only, as it uses the static class internally.
	/// </summary>
	[TestFixture]
	public class TestSymmetricCryptography
	{
		private SymmetricCryptography<TripleDESCryptoServiceProvider> crypto3DES;
		private SymmetricCryptography<DESCryptoServiceProvider> cryptoDES;
		private SymmetricCryptography<RijndaelManaged> cryptoRijndael;
		private SymmetricCryptography<RC2CryptoServiceProvider> cryptoRC2;

		private string key;
		private string iv;

		[TestFixtureSetUp]
		public void Setup()
		{
			key = Guid.NewGuid().ToString();
			iv = Guid.NewGuid().ToString().Substring(0, 3);
			crypto3DES = new SymmetricCryptography<TripleDESCryptoServiceProvider>(key, iv);
			cryptoDES = new SymmetricCryptography<DESCryptoServiceProvider>(key, iv);
			cryptoRijndael = new SymmetricCryptography<RijndaelManaged>(key, iv);
			cryptoRC2 = new SymmetricCryptography<RC2CryptoServiceProvider>(key, iv);
		}

		[Test]
		public void TestDESString()
		{
			TestString(cryptoDES);
		}

		[Test]
		public void Test3DESString()
		{
			TestString(crypto3DES);
		}

		[Test]
		public void TestRijndaelString()
		{
			TestString(cryptoRijndael);
		}

		[Test]
		public void TestRC2String()
		{
			TestString(cryptoRC2);
		}

		[Test]
		public void TestDESStream()
		{
			TestStream(cryptoDES);
		}

		[Test]
		public void Test3DESStream()
		{
			TestStream(crypto3DES);
		}

		[Test]
		public void TestRijndaelStream()
		{
			TestStream(cryptoRijndael);
		}

		[Test]
		public void TestRC2Stream()
		{
			TestStream(cryptoRC2);
		}

		[Test]
		public void TestDESStringStream()
		{
			TestStringStream(cryptoDES);
		}

		[Test]
		public void Test3DESStringStream()
		{
			TestStringStream(crypto3DES);
		}

		[Test]
		public void TestRijndaelStringStream()
		{
			TestStringStream(cryptoRijndael);
		}

		[Test]
		public void TestRC2StringStream()
		{
			TestStringStream(cryptoRC2);
		}

		[Test]
		public void TestSameKeyIV()
		{
			SymmetricCryptography<TripleDESCryptoServiceProvider> crypto2 =
				new SymmetricCryptography<TripleDESCryptoServiceProvider>(key, iv);
			string text = Guid.NewGuid().ToString();
			string encrypted = crypto3DES.Encrypt(text);
			string encrypted2 = crypto2.Encrypt(text);
            StringAssert.Equals( encrypted, encrypted2 );

			// should be able to decrypt with either crypto
			string decrypted = crypto3DES.Decrypt(encrypted2);
            StringAssert.Equals( text, decrypted );
		}

		[Test]
		[ExpectedException(typeof (CryptographicException))]
		public void TestDifferentAlgorithms()
		{
			string text = Guid.NewGuid().ToString();
			string encrypted = cryptoDES.Encrypt(text);
			string decrypted = crypto3DES.Decrypt(encrypted);
		}

		#region Helper for tests

		private void TestString<T>(SymmetricCryptography<T> crypto) where T : SymmetricAlgorithm, new()
		{
			string text = Guid.NewGuid().ToString();
			string encrypted = crypto.Encrypt(text);
			string decrypted = crypto.Decrypt(encrypted);
            StringAssert.Equals( text, decrypted );
		}

		private void TestStream<T>(SymmetricCryptography<T> crypto) where T : SymmetricAlgorithm, new()
		{
			Guid guid = Guid.NewGuid();
			using (MemoryStream ms = new MemoryStream(guid.ToByteArray()))
			{
				using (MemoryStream s = new MemoryStream())
				{
					crypto.Encrypt(ms, s);
					MemoryStream s2 = new MemoryStream(s.ToArray());
					using (MemoryStream ds = new MemoryStream())
					{
						crypto.Decrypt(s2, ds);
						ds.Position = 0;
						ms.Position = 0;
						byte[] output = ds.ToArray();
						Assert.AreEqual(guid.ToByteArray().Length, output.Length);
						for (int i = 0; i < output.Length; i++)
						{
							Assert.AreEqual(guid.ToByteArray()[i], output[i]);
						}
					}
				}
			}
		}

		private void TestStringStream<T>(SymmetricCryptography<T> crypto) where T : SymmetricAlgorithm, new()
		{
			string text = Guid.NewGuid().ToString();
			MemoryStream ms = new MemoryStream();
			crypto.EncryptToStream(text, ms);
			MemoryStream ms2 = new MemoryStream(ms.ToArray());
			string decrypted = crypto.DecryptFromStream(ms2);
            StringAssert.Equals( text, decrypted );
		}

		#endregion

		/*  These methods don't work, although it looks like they should.  Will work on later.
        public void TestArray<T>( SymmetricCryptography<T> crypto ) where T : SymmetricAlgorithm, new()
        {
            Guid guid = Guid.NewGuid();
            byte[] encrypted = crypto.Encrypt( guid.ToByteArray() );
            byte[] decrypted = crypto.Decrypt( encrypted );
            Assert.AreEqual( guid.ToByteArray(), decrypted );
        }
         */
	}
}